package log

import (
	"fmt"
	"github.com/sirupsen/logrus"
	"os"
	"path/filepath"
	"runtime"
	"time"
)

var GlobalLogger *logrus.Logger
var MonitorLogger *logrus.Logger

// InitLogConfig 初始化日志配置
func InitLogConfig() {
	initGlobalLogger()
	initMonitorLogger()
}

// initGlobalLogger 初始化日志配置
func initGlobalLogger() {
	time.Sleep(time.Duration(1) * time.Second)
	// 创建、追加、读写，777，所有权限
	f, err := os.OpenFile("program-"+time.Now().Format("2006-01-02-15-04-05")+".log", os.O_CREATE|os.O_APPEND|os.O_RDWR, os.ModePerm)
	if err != nil {
		os.Exit(-1)
	}
	GlobalLogger = logrus.New()
	GlobalLogger.SetOutput(f)
	GlobalLogger.SetReportCaller(true) // 开启行号显示
	GlobalLogger.SetFormatter(&logrus.JSONFormatter{
		CallerPrettyfier: func(frame *runtime.Frame) (string, string) {
			fileName := filepath.Base(frame.File)
			return "", fmt.Sprintf("%s:%d", fileName, frame.Line)
		},
	})
	GlobalLogger.Info("初始化GlobalLogger - 成功")

}

// initMonitorLogger 初始化日志配置
func initMonitorLogger() {
	time.Sleep(time.Duration(1) * time.Second)
	// 创建、追加、读写，777，所有权限
	f, err := os.OpenFile("monitor-"+time.Now().Format("2006-01-02-15-04-05")+".log", os.O_CREATE|os.O_APPEND|os.O_RDWR, os.ModePerm)
	if err != nil {
		os.Exit(-1)
	}
	MonitorLogger = logrus.New()
	MonitorLogger.SetOutput(f)
	MonitorLogger.SetReportCaller(true) // 开启行号显示
	MonitorLogger.SetFormatter(&logrus.JSONFormatter{
		CallerPrettyfier: func(frame *runtime.Frame) (string, string) {
			fileName := filepath.Base(frame.File)
			return "", fmt.Sprintf("%s:%d", fileName, frame.Line)
		},
	})
	MonitorLogger.Info("初始化MonitorLogger - 成功")
}
